/* SREhttp/2 link-checker addon, ver 1.20

This uses the CheckLink "os/2 standalone" program to do the actual work.  
It launches CHEKLINK.CMD as a SRE2002 "task", and feeds this task the names of
several temporary files. 

Notes:
  * The temporary files used (all of which are created in the
    TEMP directory of the ADDON\CHEKLINK directory)
       parameters file:  CHEK_SR1.nnn 
       output file: CHEK_SR2.nnn
       status file: CHEK_SR3.nnn
     The output file is what will contain the actual results of the link checking.
     The status file contains run time status information.

     These files are NOT deleted when CHEK_SRE (or CHEKLINK.CMD) complete.
     Instead, they will be deleted when chek_sre needs space.

  * Security note                               

          Since the temporary files are NOT deleted when the task finishes.
          a clever hacker could discover what other queries
          were made by other clients, and also see the results.

         If this is a problem, you'll need to carefully limit access
         to the CHEK_SRE addon.

  * CHEKLINK.CMD contains several user-changeable parameters.
    HOWEVER, in a given SRE2002 session...
       once you run CHEK_SRE the first time, the version of CHEKLINK.CMD
       is loaded into macrospace, and used for all subsequent invocations of
       CHEK_SRE.
    Thus, if you change CHEKLINK.CMD, you'll need to restart SRE2002.

     
*/

cheklink:


/*********          BEGIN USER CONFIGURABLE PARAMETERS              ********/
/* these can be used to tune performance and modify the output.             */

/* used in <BODY back> element  */ 
back_1='bgcolor="#668a78"'   

/* directory to find CHEKLINK.CMD; and under which to find the LINKFILE
   subdirectory (for  storing "link" (.STM) files).
   If not specified, use ADDON\CHEKLINK subdirectory of SREHTTP2 */
cheklink_dir=''

/*   NEED_PRIVS is a set of "privileges" that permit creation of a swish
     index.  That is, to create a swish index, the client must have ONE 
     of these privileges (not all of, but at least one of). 
     If she doesn't have one of these privileges, an authorization request 
     will be returned.
     If need_privs='', or  contains an *, then need_privs is ignored (open access)
*/
NEED_PRIVS='SUPERUSER CHEK_SRE'

/*** The remaining parameters control standalone mode screen output  
     You may need to change them (especially the "frame" characters")
     if you are using a non-english or non "latin 1" code page        */

/* display this "name of the program"     */
topmess="CheckLink ver 1.20 "


/**************** END USER CONFIGURABLE PARAMETERS **********************/


parse arg list,servername,verb,tempfile,,
          prog_file,reqnum,verbose,client_ip,privset,,
          uri,host_nickname,id_info,aiter,attribs


servername=strip(servername)
signal on error name iserr ; signal on novalue name iserr 
signal on syntax name iserr
signal on failure name iserr ;

 
/*  === default values of parameters */

ascgi=0         /* type of calls to cheklink2 */

base_uri=0      /* An "http://..." abbreviation specifying what html documents should be 
                    processed for links. If 0,  the BASEONLY variable is used. */

baseonly=1      /* What html documents to process for links. Used if BASE_URI=0
                   0 = relative to root directory of the STARTER_URL's site, or
                       to the root directory of the site noted in a
                       <BASE element in the STARTER_URL. 
                   1 = use directory of the starter-url, or to the directory of a
                       <BASE element in the STARTER-URL.       */

/* space delimited list of wildcardable selectors to NOT check */
exclusion_list='!*  *?* *MAPIMAGE/* */OLD* */ARCHIVE* CGI*'  

cheklink_htm=''     /* where to write results. if blank, auto generate a name (recommended) */
include_cheklnk2=2  /* Include links to CHEKLNK2 linkage-file-reader. 0=no, 1=yes */

link_file=''     /* Create a .STM (linkages) file. 0=Do not create. ''=auto generate */
make_descrip=2   /* Generate descriptions: 1=no, 2=if text/html, 3=if text/html or text/plain */
outtype='ALL'    /* tables to generate: list of OK NOSITE NOURL OFFSITE EXCLUDED (or ALL) */
queryonly=0      /* 1=just query, do not GET, links (do NOT process any HTMLS) */
siteonly=0       /* 1=no query on off-site urls (note that "query" means "issue a HEAD request")*/
starter_url=''   /* REQUIRED: the starter url */
treename=''      /* default descriptive name */
user_pwd=''      /* default username password */
verbose=1        /* verbosity of status messages: 1=little, 2=some, 3=too much */

/* ======= End of default parameters */

crlf='0d0a'x

/* check privileges .... */
if need_privs<>'' & pos('*',need_privs)=0 then do
  if privset='' then do
    if aiter='' then return 'PRIVS CHEK_SRE addon requires special privileges if you want to generate an index (administrators: to disable this requirement, see CHEKLINK\CHEK_SRE.CMD in your SREHTTP2\ADDON directory)'
    return 'AUTH CHEK_SRE addon requires special privileges if you want to generate an index (administrators: to disable this requirement, see CHEKLINK\CHEK_SRE.CMD in your SREHTTP2\ADDON directory)'
  end
  astat=sreh2_check_privs(need_privs,privset)
  if astat=0 then do
    if aiter='' then return 'PRIVS CHEK_SRE addon requires special privileges if you want to generate an index (administrators: to disable this requirement, see CHEKLINK\CHEK_SRE.CMD in your SREHTTP2\ADDON directory)'
    return 'AUTH CHEK_SRE addon requires special privileges if you want to generate an index (administrators: to disable this requirement, see CHEKLINK\CHEK_SRE.CMD in your SREHTTP2\ADDON directory)'
  end
end

/* file and directories */
if cheklink_dir=0 | cheklink_dir='' then do
   cheklink_dir=sre_value('H2_ADDON_DIR_DEFAULT',,'SRE')
   cheklink_dir=strip(cheklink_dir,,'\')||'\CHEKLINK'
end

cheklink_dir=strip(cheklink_dir,'t','\')
if dosisdir(cheklink_dir)=0 then do
  return 'String Sorry. The CHEKLINK_DIR directory (<tt>'||cheklink_dir||'</tt>) does not exist '
end

cheklink_dir=strip(cheklink_dir,'t','\')||'\'

LINKFILE_dir=cheklink_dir||'LINKFILE'
if dosisdir(linkfile_dir)=0 then do
  return 'String Sorry. The LINKFILE_DIR directory (<tt>'||linkfile_dir||'</tt>) does not exist '
end
LINKFILE_dir=cheklink_dir||'LINKFILE\'

cheklink_cmd=cheklink_dir||'CHEKLINK.CMD'
if stream(cheklink_cmd,'c','query exists')='' then do
  return 'String Sorry. The CHEKLINK.CMD file (<tt>'||cheklink_CMD||'</tt>) does not exist '
end

chektemp_dir=cheklink_dir||'TEMP'
if dosisdir(chektemp_dir)=0 then do
  return 'String Sorry. The CHEKTEMP_DIR directory (<tt>'||chektemp_dir||'</tt>) does not exist '
end

param_file=sre_tempfile('CHEK_SR1.??','?',chektemp_dir)
output_file=sre_tempfile('CHEK_SR2.??','?',chektemp_dir)
status_file=sre_tempfile('CHEK_SR3.??','?',chektemp_dir)

instuff=' '
doview=0

do until list=''
   parse var list a1 '&' list
   parse var a1 avar '=' aval ; tavar=translate(avar)
   aaval=sre_packur(translate(aval,' ','+'))
   select
     when tavar='URL' | tavar='STARTER_URL' then starter_url=aaval
     when tavar='BASE_URI' then base_uri=aaval
     when abbrev(tavar,'BASE_QU')=1 then do
        aaval=strip(aaval)
        if wordpos(aaval,'0_0 1_0 0_1 1_1')>0 then
           parse var aaval baseonly '_' queryonly
     end 
     when abbrev(tavar,'BASE')=1 then baseonly=is_yes_no(aaval,baseonly)
     when abbrev(tavar,'QUERY')=1 then queryonly=is_yes_no(aaval,queryonly)
     when abbrev(tavar,'SITE')=1 then siteonly=is_yes_no(aaval,siteonly)
     when abbrev(tavar,'EXCLUS')=1 then  exclusion_list=aaval
     when abbrev(tavar,'OUTPUT')=1 then output_file=strip(aaval)
     when abbrev(tavar,'VIEW')=1 then doview=strip(aaval)
     when abbrev(tavar,'STATUS')=1 then status_file=strip(aaval)
     when abbrev(tavar,'OUTTYPE')=1 then do 
          if aaval<>'' & aaval<>0 then outtype=translate(aaval)
     end
     when (abbrev(tavar,'LINKFILE')+abbrev(tavar,'LINK_FILE'))>0 then link_file=strip(translate(aaval))
     when abbrev(tavar,'NAME')>0 then treename=aaval
     when abbrev(tavar,'RESULT')=1 then result_file=aaval
     when abbrev(tavar,'DESCRIP')=1 then do
          make_descrip=wordpos(aaval,'1 2 3')
          if make_descrip=0 then make_descrip=1  /* unknown option means "no descrips */
     end 
     otherwise nop
  end
end


/* might be view mode? */
if doview>0 then do
  call do_view
  aa=result
  return aa
end

/* not view mode .... */

if link_file<>0 & link_file<>'' then do
   if pos(':',link_file)+pos('\',link_file)>0 then do
     return 'String Sorry. You can not specify a directory when creating a Link File '
   end
   link_File=linkfile_dir||link_file
   if pos('.',link_File)=0 then link_File=link_File'.STM'
end 
  

if pos('?',link_file)>0 then do
   link_File=sre_tempfile(link_file)
end

aurl=starter_url
if aurl='' then do      /* Starter-url is REQUIRED */
  return 'String Sorry. You must specify a <b>starter-url </b><p><a href="javascript:history.go(-1)">Back</a>'
end


taurl=translate(aurl)
if abbrev(taurl,'HTTP://')+abbrev(taurl,'HTTPS://')=0 then do
   if pos('/',aurl)>0 then do   
      parse var taurl eek1 '/' .
      if pos('.',eek1)>0 then do  /* is seems before first / is a domain name */
         aurl='http://'||aurl
      end 
      else do           /* preceding first / seems to be a directory name */
         aurl='http://'||sre_servername()||'/'||strip(aurl,'l','/')
      end
   end
   else do              /* no / in starterurl */
      parse var aurl . '.' goo1
      if pos('.',goo1)>0 then do       /*  probably  a domain name */
         aurl='http://'||aurl||'/'
      end 
      else do           /* preceding first / seems to be a directory name */
         aurl='http://'||sre_servername()||'/'||strip(aurl,'l','/')
      end
   end 
   taurl=translate(aurl)
end
parse var aurl . '//' server '/' request
if server='' then server=sre_servername()
server=strip(server)
if request='' then request='/'

foo='<html><head><title>For CheckLink of ' server ' </title> ' crlf 
dabody=foo'</head> <body ' back_1'>'


ofile=filespec('n',output_File)
parse var uri u1 '?' .
u2=u1||'?VIEW=1&output='||filespec('n',output_file)
u3=u1||'?VIEW=2&status='||filespec('n',status_file)

dabody=dabody||crlf||'<h3 align="center">Using CheckLink to create a web tree </h3>'
dabody=dabody||crlf||'<table border=1><tr><td><em>'
dabody=dabody||crlf||' CheckLink will use the parameters listed <br> below to create a web tree.</em>'
dabody=dabody||crlf||'<td><menu type="circle"><li >View the: '
dabody=dabody||crlf||'<a href="'||u2||'">final results</a> <em>depending on how many links are to be checked, this may not be ready for a while)</em>'

dabody=dabody||crlf||'<li>Or, you can view the '
dabody=dabody||crlf||'<a href="'||u3||'">current status</a>'
dabody=dabody||crlf||'</menu> '
dabody=dabody||crlf||'</td></table>'


noyes.0='NO' ; noyes.1='YES'
is_descrip.1='None created' ; is_descrip.2='text/html only' ;is_descrip.3='text/html &amp; text/plain '
intro2=' <br><h3>Parameters</h3>'crlf'<ul>' 

sayw=''
sayw='(* are wildcards)'
intro2=intro2||,
    ' <li>Descriptive Name= 'treename || crlf 

if base_Uri=0 then
   intro2=intro2||' <li>  BASEONLY  = ' noyes.baseonly '&nbsp; &nbsp; (YES= only process links in htmls under <em>base-url</em>)'||crlf 
else
   intro2=intro2||' <li>  BASE_URI  = <u>'||base_uri||'</u> &nbsp; &nbsp; '||crlf

intro2=intro2||' <li>  QUERYONLY = ' noyes.queryonly '&nbsp; &nbsp; (YES= query, but do not GET, links)' crlf ,
    ' <li>  SITEONLY  = ' noyes.siteonly ' &nbsp; &nbsp;(YES= do <em>not </em> query off-site links)'crlf ,
    ' <li>DESCRIPTIONS= ' is_descrip.make_descrip || crlf ,
    ' <LI>EXCLUSION_LIST = <b>' exclusion_list '</b> &nbsp; &nbsp; '||sayw||crlf 
intro2=intro2||,
    ' <li>    OUTTYPE = <b>' outtype '</b>&nbsp; &nbsp; (types of results to report) 'crlf 
intro2=intro2'<li> OutputFile = 'output_file ||crlf
intro2=intro2'<li> Verbosity  = '||word('Quiet Normal Verbose VeryVerbose',verbose)||crlf
if link_file<>'' & link_file<>0 then do
     intro2=intro2'<li>   LinkFile = '||filespec('n',link_file)||crlf
     if include_cheklnk2=1 then do
        intro2=intro2'<li>   Include links to CHEKLNK2 = '||noyes.include_cheklnk2||crlf
        intro2=intro2'<br>                 call as CGI-BIN= '||noyes.ascgi||crlf
     end
end
intro2=intro2||'</ul>' crlf 
 
intro2=intro2'    <b>Starter URL=</b> ' aurl
intro2=intro2'<br>            &nbsp;&nbsp;&nbsp;&nbsp;<tt>server</tt>=<u>' server '</u>, <tt>selector</tt>=<u>' request '</u><br>'

dabody=dabody||'0d0a'x||intro2

dabody=dabody||crlf||'</body></html>'


call save_params param_file

astat0=sre_task('EXIST','CHEKLINK.CHECKL')
if astat0<>0 then do
/* define the sre-task family of CHECKL, and CHEKLINE within it */
  astat=sre_task('FAMILY','CHECKL','1,10000,10')
  astat=sre_task('DEFINE','CHEKLINK.CHECKL',,CHEKLINK_CMD)  
  parse var astat a1 a2 
  if datatype(a1)<>'NUM' then do
       return 'String Sorry, unable to launch CHEKLINK daemon: 'astat
  end 
end 

/* now launch the daemon */
astat=sre_task('LAUNCH','CHEKLINK.CHECKL',,status_file||','||param_file||','||cheklink_dir)
if abbrev(translate(astat),'ERROR')=1 then
   dabody=dabody||crlf||'<pre>'||astat||'</pre>'

if abbrev(translate(astat),'PENDING')=1 then
   dabody=dabody||crlf||'<pre>NOTE: The CheckLink utility is currently being used. It might take a while before your request is processed.</pre>'

dabody=dabody||crlf||'</body></html>'

return 'Var type text/html '||'0d0a'x||dabody



/****************/
/* show status or final output */
do_view:

if doview=1 then do             /* 1 means "view output " */
  ofile=filespec('n',output_file)
  dd=chektemp_dir
  ouse=strip(dd,'t','\')||'\'||ofile
   if ouse='' then do
     return 'String Sorry. Output file (<tt>'||ofile||'</tt>) not available. '||,
           ' Perhaps CheckLink is still working...<p>'||,
           ' <a href="javascript:history.go(-1)">Back</a>'
  end 
  stuff=sre_read_file(ouse,2,5)
  if stuff='' then 
       return 'String Sorry, unable to read results file. Please try again later '
  return 'var type text/html  '||'0d0a'x||stuff
end
    
/* else, status file */
ofile=filespec('n',status_file)
dd=chektemp_dir
ouse=strip(dd,'t','\')||'\'||ofile
if stream(ouse,'c','query exists')='' then do
     return 'String Sorry. Status for CheckLink is not available. '||,
     ' Perhaps CheckLink completed ...<hr>'||,
     ' <a href="javascript:history.go(-1)">Back</a>'
end 

stuff=sre_read_file(ouse,2,5)
if stuff='' then 
    return 'String Sorry, unable to read status file. Please try again later '
return 'var type text/plain  '||'0d0a'x||stuff





/*=============================================*/


/******/
iserr:
call pmprintf('Check_SRE error at 'sigl ','rc)
call pmprintf(" C "||condition('C'))
call pmprintf('I '||condition('i'))
call pmprintf('s '||condition('s'))
call pmprintf('d '||condition('d'))

say ='CheckLink error  'sigl ','rc

exit

/***************************/
save_params:
parse arg nuinfile
cmdlist='STARTER_URL TREENAME USER_PWD  INCLUDE_CHEKLNK2  CHEKLINK_HTM  '||,
        'BASE_URI BASEONLY SITEONLY QUERYONLY MAKE_DESCRIP '||,
        'EXCLUSION_LIST OUTTYPE OUTPUT_FILE LINK_FILE VERBOSE ASCGI'

if stream(nuinfile,'c','query exists')<>' ' then do
  nuinfile=sysfiledelete(nuinfile)
end

call lineout nuinfile,'; CheckLink Parameters file created '||time('n')||'  '||date('n')
do mm=1 to words(cmdlist)
   avar=strip(word(cmdlist,mm))
   aval=value(avar)
   call lineout nuinfile,avar '=' aval
end 
call lineout nuinfile
return 1

/***************/
/* return 0 for no, 1 for yes, default otherwise */
is_yes_no:procedure
parse arg aval,def
tdef=strip(translate(aval))
if wordpos(tdef,'Y YES 1')>0 then return 1
if wordpos(tdef,'N NO 0')>0 then return 0
return def

